<?php

namespace Drupal\config_import;

use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\features\ConfigurationItem;
use Drupal\features\FeaturesAssignerInterface;
use Drupal\features\FeaturesManagerInterface;
use Drupal\config_update\ConfigRevertInterface;

/**
 * Defines the config features importer.
 */
class ConfigFeaturesImporterService {

  /**
   * The features manager.
   *
   * @var array
   */
  protected $featuresManager;

  /**
   * The package assigner.
   *
   * @var array
   */
  protected $assigner;

  /**
   * The config reverter.
   *
   * @var \Drupal\config_update\ConfigRevertInterface
   */
  protected $configRevert;

  /**
   * The logger factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $logger_factory;

  /**
   * Constructs a ConfigFeaturesImporterService object.
   *
   * @param \Drupal\features\FeaturesManagerInterface $features_manager
   *   The features manager.
   * @param \Drupal\features\FeaturesAssignerInterface $assigner
   *   The features assigner.
   * @param \Drupal\config_update\ConfigRevertInterface $config_revert
   *   The config reverter.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory.
   */
  public function __construct(FeaturesManagerInterface $features_manager, FeaturesAssignerInterface $assigner, ConfigRevertInterface $config_revert, LoggerChannelFactoryInterface $logger_factory) {
    $this->featuresManager = $features_manager;
    $this->assigner = $assigner;
    $this->configRevert = $config_revert;
    $this->logger = $logger_factory->get('config_update');
  }

  /**
   * {@inheritdoc}
   */
  public function import($featurename) {
    if (empty($featurename)) {
      return;
    }
    $current_bundle = $this->assigner->applyBundle();
    $packages = $this->featuresManager->getPackages();

    if (!empty($featurename) && empty($packages[$featurename])) {
      $this->logger->error(t('Feature @name does not exist.', ['@name' => $featurename]));
      return;
    }
    elseif (!empty($featurename)) {
      $packages = [$packages[$featurename]];
    }
    else {
      $packages = $this->featuresManager->filterPackages($packages, $current_bundle->getMachineName());
    }

    $items = [];
    foreach ($packages as $package) {
      if ($package->getStatus() != FeaturesManagerInterface::STATUS_NO_EXPORT) {
        $missing = $this->featuresManager->reorderMissing($this->featuresManager->detectMissing($package));
        $overrides = $this->featuresManager->detectOverrides($package, TRUE);
        $items += $missing + $overrides;
      }
    }

    $this->assigner->assignConfigPackages();
    $config = $this->featuresManager->getConfigCollection();
    if (empty($items)) {
      return;
    }
    foreach ($items as $config_name) {
      if (isset($config[$config_name])) {
        $item = $config[$config_name];
        $type = ConfigurationItem::fromConfigStringToConfigType($item->getType());
        $this->configRevert->revert($type, $item->getShortName());
      }
      else {
        $item = $this->featuresManager->getConfigType($config_name);
        $type = ConfigurationItem::fromConfigStringToConfigType($item['type']);
        $this->configRevert->import($type, $item['name_short']);
      }
      $this->logger->info(t('Imported @name', ['@name' => $config_name]));
    }
  }

}
